//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	DiracSync
// Auteur			:	PERRICHON Pierre
// Date				:	07/01/2021
// Version			:	1.0
//
// Sujet			: 	Génération de peigne de Dirac synchronisé sur une date d'amorçage
//
// Paramétrage		:	Instant de départ (s)
//						Amplitude
//						Période
//						Durée de l'impulstion (mettre typiquement 100 ms)
//						Impulsion sysmétrique (1) ou non (0) à chaue période
//
// Autres données	:	
//						
//						
// Sorties			:	Peigne de Dirac
//						
//						
//						
// Interface IHM	:	DiracSync.sci
// Code objet		;	DiracSyncC.c 
//
// Programme de test:	DiracSync.zos, GeneBF.zcos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=DiracSync(job, sciblk, e1)

	s1= []; s2 = [];// arguments requis mais non utilisés 
	typ=[];

	Module="DiracSync";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
	
	select job
	
		case "set" then
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs
			model=sciblk.model;
			
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end
									
			labels=["Instant de départ (s) tDirac";...
			        "Amplitude ADirac";...
					"Période (s) PDirac";...
					"Durée Impulsion (s) DDirac";...
					"Symérique (0 ou 1) SDirac"];
					
			messerrPDirac="La constante de temps PDirac doit être strictement positive";					
			messerrDDirac="La durée du Dirac DDirac doit être strictement positive";					
			messerrSDirac="La valeur de SDirac est incorrecte (0 ou 1)";		
					
			if getlanguage()=="en_US" then		
					
				labels=["Start time (s) tDirac";...
			        "Amplitude ADirac";...
					"Period (s) PDirac";...
					"Pulse Duration(s) DDirac";...
					"Symmetry (0 or 1) SDirac"];
					
				messerrPDirac="The PDirac time constant must be strictly positive";					
				messerrDDirac="The duration of the Dirac DDirac must be strictly positive";					
				messerrSDirac="SDirac value is incorrect (0 or 1)";					
					
			end
			
			while %t do
			
				[ok,tDirac,ADirac,PDirac,DDirac,SDirac,exprs]=scicos_getvalue("Set DiracSync",labels,...
												   list("vec",1,"vec",1,"vec",1,"vec", 1,"vec",1),exprs)
				if ~ok then return; end
				
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							if PDirac<=0 then DefExp=[DefExp;messerrPDirac]; end
							if DDirac<=0 then DefExp=[DefExp;messerrDDirac]; end
							if PDirac<=DDirac then DefExp=[DefExp;"PDirac > DDirac"]; end
							if ~or(SDirac==[0;1]) then DefExp=[DefExp;messerrSDirac]; end
							
							if DefExp==[] then break; end
							
							SegPalError(1, Module, DefExp);
				end
				
			end
			
			graphics.exprs=exprs;
			model.rpar=[tDirac; ADirac; PDirac; DDirac ;SDirac];	
					
			sciblk.graphics=graphics;
			sciblk.model=model
					
			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=middle";
						"fillColor=#abcdee";
						"fontColor=blue";
						"displayedLabel=DiracSync"];
			sciblk.graphics.style = strcat(style_properties,";");
				
		
		case "define" then
			tDirac=0			// Instant de départ
			ADirac=0.1		// Ampitude initiale
			PDirac=100			// Période initiale
			DDirac=0.2		// Durée de l'impulsion
			SDirac=1			// Impulsion symétrique
			
			model=scicos_model()
			model.sim=list("DiracSyncC",4)
			model.in=[];model.in2=[];
			model.out=1;model.out2=[1]
			
			rpar=[tDirac;ADirac; PDirac; DDirac; SDirac];	
			exprs = [sci2exp(tDirac);sci2exp(ADirac);sci2exp(PDirac);sci2exp(DDirac);sci2exp(SDirac)];

			model.blocktype="c"
			model.dep_ut=[%f %t]

			sciblk=standard_define([3. 2.],model,exprs);
		
			style_properties = ["blockWithLabel";
                        "verticalLabelPosition=middle";
                        "verticalAlign=middle";
						"fillColor=#abcdee";
						"fontColor=blue";
						"displayedLabel=DiracSync"];
			sciblk.graphics.style = strcat(style_properties,";");

	end
	
endfunction
