//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	GainVar
// Auteur			:	PERRICHON Pierre
// Date				:	07/07/2017
// Version			:	1.0
//
// Sujet			: 	Répartiteur de jet - Rampe de ralliement lors des commutations de jets d'ue machine PELTON multi-jets
//						
//
// Paramétrage		:	Temps de commutation (s)
//						Valeur initiale du gain variable à t=0
//
// Autres données	:	Le gain objectif doit être supérieur ou égal à 1
//						Un gain nul provoque une erreur de synchronisation sur Xcos !
//						
//						
// Sorties			:	Rampe sur le gain variable
//						
//						
//						
// Interface IHM	:	GainVar.sci
// Code objet		;	GainVarC.c 
//
// Programme de test:	GainVar.zos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=GainVar(job, sciblk, e1)

	s1= []; s2 = [];// arguments requis mais non utilisés 
  
	Module="GainVar";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
   
	select job 
  
		case "set" then 
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs;
			model=sciblk.model;
			rpar=model.rpar;
			
			Titre="Paramètres de GainVar :"
			
			labels=["Temps de commutation (s)";...
			        "Valeur initiale du gain variable GVi à t=0"];
										
			meserrTau="Le temps de commutation doit être strictement positif"
			meserrX0="Le ou les gains initiaux doivent être strictement positifs"

			if getlanguage()=="en_US" then	

				Titre="GainVar settings:"			
					
				labels=["Switching time (s)";...
			        "Initial value of the variable gain GVi at t=0"];
					
				meserrTau="The switching time must be strictly positive"
				meserrX0="The initial gain(s) must be strictly positive"			
					
			end
					
			while %t do
			
				[ok,Tau,x0,exprs]=scicos_getvalue(Titre,labels,...
									list("vec",1,"vec",-1),exprs);
					
				if ~ok then return; end

				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							if Tau<=0 then DefExp=[DefExp;meserrTau]; end
							if ~and(x0>=0) then DefExp=[DefExp;meserrX0]; end	

							if DefExp==[] then break; end							

							SegPalError(1, Module, DefExp);
				end				
				
				//Analyse du paramétrage
				//----------------------			
			
				if (Tau<=0) then
                    message("La constante de temps du filtre doit être strictement positive")
                    ok=%f
				end
				
			end
			
			n = size(x0,'*');
					
			in  = list([n 1],1);					
			graphics.in_label=[""];
					
			out = list([n 1;n 1],[1 1]);
										
			evtin = [];		//	aucun port dentrée d'activation
			evtout = []; 	//	aucun port de sortie d'activation
																									
			//Redessiner l'interface graphique en fonction de la valeur d'initialisation
			//--------------------------------------------------------------------------
			[model,graphics,ok] = set_io(model, graphics, in, out, evtin, evtout)
				
			// Terminé ==> transfert des structures et validation
			//---------------------------------------------------
			model.state = x0;		// Valeur initiale du filtre passe-bas
			model.rpar=[1/Tau];		// Constante de temps du filtre passe-bas
							
			sciblk.model = model;
			graphics.exprs = exprs;
			sciblk.graphics = graphics;
							
			//Affichage de la constante de temps du filtre dans la boîte fonctionnelle
			//------------------------------------------------------------------------
			Taudisplay=string(exprs(1))

			style_properties = ["blockWithLabel";
									"verticalLabelPosition=middle";
									"verticalAlign=top";
									"align=center";
									"fontColor=blue";
									"fontSize=12";
									"displayedLabel=GainVar <br> <br>" + "t = " + string(Taudisplay) + " (s)";
									"fillColor=#abcdee";
									"shadow=true"];									
			sciblk.graphics.style = strcat(style_properties,";");
			sciblk.graphics.out_label=["gv", "sr"]			
							
		
		case "define" then 
	
			Tau=1;		// Constante de temps de l'intégrateur (s)
			x0=0;		// Valeur initiale nulle de l'intégrateur			
		
			model = scicos_model();
			model.state = x0;
			model.sim = list("GainVarC",4);
			
			model.intyp = 1;
			model.in = 1; model.in2 = 1; 		//une entrée scalaire
			
			model.outtyp = [1;1];
			model.out = [1;1]; 
			model.out2 = [1;1];					//2 sorties scalaire

			model.rpar=[1/Tau];
			model.dep_ut = [%f %t];
			
			exprs = [sci2exp(Tau);sci2exp(x0)];
			
			sciblk = standard_define([7 4],model,exprs);
			
			style_properties = ["blockWithLabel";
									"verticalLabelPosition=middle";
									"verticalAlign=top";
									"align=center";
									"fontColor=blue";
									"fontSize=12";
									"displayedLabel=GainVar <br> <br>" + "t = " + string(Tau) + " (s)";
									"fillColor=#abcdee";
									"shadow=true"];									
					
			sciblk.graphics.style = strcat(style_properties,";")	
			sciblk.graphics.out_label=["gv", "sr"]			
	end
  
endfunction