//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	IntSPE
// Auteur			:	PERRICHON Pierre
// Date				:	01/05/2014
// Version			:	1.0
//
// Sujet			: 	Intégrateurs avec désaturateur rapide et saturations externes
//
// Paramétrage		:	Constante de temps Ti de l'intégrateur (s)
//						Valeur initiale de l'intégrateur à t=0
//						Constante de temps du désaturateur(s)
//						Avec ou sans saturations externes (0 ou 1)
//
// Autres données	:	
//						
//						
// Sorties			:	Intégrateurs avec saturation
//						
//
//						
// Interface IHM	:	IntSPE.sci
// Code objet		;	IntSPEC.c 
//
// Programmes de test:	IntSPE.zos
//
//------------------------------------------------------------------------------------------------------------------------
function [sciblk, s1, s2]=IntSPE(job, sciblk, e1)

	s1= []; s2 = [];	// arguments requis mais non utilisés 
  
	Module="IntSPE";
	[linenum, fun] = where()
	editSegPalBlock=~or(fun == "do_eval")	// Détertime si mode simulation ou édition du composant par double clic
  
  
  reinit=1			// Forçage réinitialisation
  
	select job 
  
		case "set" then 
		
			graphics=sciblk.graphics;
			exprs=graphics.exprs;
			model=sciblk.model;
			
			// Analyse préliminaire des expressions
			//-------------------------------------
			[bdiag,DefExp]=CheckExprs(exprs) 		
			if ~bdiag then 
						// Certaines variables ne sont pas définies
						//-----------------------------------------
						if SegPalError(2, Module, DefExp)==2 then return; end
			end			

			Titre="Paramètres de l''intégrateur IntSPE"		
			
			labels=["Constante de temps de l''intégrateur (s)";...
			        "Valeur initiale de l''intégrateur à t=0";...
					"Constante de temps du désaturateur (s)";...
					"Avec saturation dynamique (1:oui - 0:non)"];
					
			meserrTau="La constante de temps de l''intégrateur doit être strictement positive";
			meserrTDesat="La constante de temps du désaturateur doit être supérieure ou égale à 0.1 s";
			
			if getlanguage()=="en_US" then

			Titre="IntSPE integrator settings"		
			
				labels=["Integrator time constant (s)";...
			        "Initial value of the integrator at t=0";...
					"Desaturator time constant (s)";...
					"With dynamic saturation (1:yes - 0:no)"];
					
				meserrTau="The time constant of the integrator must be strictly positive";
				meserrTDesat="The time constant of the desaturator must be greater than or equal to 0.1 s";
	
			end			
					
			while %t do
			
				[ok,Tau,x0,TDesat,sature,exprs]=scicos_getvalue(Titre,labels,...
									list("vec",1,"vec",1,"vec",1,"vec",1),exprs)
					
				if ~ok then return; end
				
				//Analyse du paramétrage
				//----------------------			
				[bdiag,DefExp]=CheckExprs(exprs)
				if ~bdiag then SegPalError(2, Module, DefExp)
					else
							DefExp=[]
							if Tau<=0 then DefExp=[DefExp;meserrTau]; end
							if TDesat<0.1 then DefExp=[DefExp;meserrTDesat]; end

							if DefExp==[] then break; end
									
							SegPalError(1, Module, DefExp);
				end				
				
			end
				
			n = size(x0,'*');	// Donne le dimensionnement variable de la boite
					
			out = [n, 1];
			ot = 1;
			evtout = []; //aucun port de sortie d'activation
					
			it = 1;
			in = [n, 1];
			graphics.in_label=[""];
			if(reinit + sature) then graphics.in_label=["in"];end;
					
			evtin = [1];	// Forçage entrée d'acivation du désaturateur
				
			// Traitement de la réinitialisation
			//----------------------------------							
			if reinit~=0 then
							reinit=1;					
							// Rajout d'1 entrée régulière de dimension 1 x 1 
							//-----------------------------------------------
							it=[it, 1];
							in=[in; 1,1];
							graphics.in_label=[graphics.in_label, "init"];

							// Un port d'entrée d'activation
							//------------------------------
							evtin = [1;1]; 							
			end
					
			//Traitement de la saturation
					//---------------------------
					if sature~=0 then
						sature=1;
						// Rajout de 2 entrées régulières de dimension 1 x 1
						//---------------------------------------------------
						it=[it, 1, 1];
						in=[in; 1,1; 1, 1];
						graphics.in_label=[graphics.in_label,"UL","LL"];
			end
					
			//Redessiner l'interface graphique en fonction de la valeur d'initialisation
			//--------------------------------------------------------------------------
			[model,graphics,ok] = set_io(model, graphics, list(in,it), list(out, ot), evtin, evtout)
				
			// Terminé ==> transfert des structures et validation
			//---------------------------------------------------
			model.state = x0;					// Valeurw initialew des intégrateurs
			model.rpar=[Tau; TDesat];			// Constante de temps des intégrateurs
			model.ipar=[reinit;sature]							
							
			sciblk.model = model;
			graphics.exprs = exprs;
			sciblk.graphics = graphics;
							
			//Affichage de la constante de temps des intégrateurs dans la boîte fonctionnelle
			//--------------------------------------------------------------------------------
			Taudisplay=string(exprs(1))+ "."
			if strcmp(Taudisplay,"1.")==0 then Taudisplay="";end;
							
			style_properties = ["blockWithLabel";
									"verticalLabelPosition=middle";
									"verticalAlign=top";
									"fontColor=black";
									"fontSize=15";
									"displayedLabel=$\frac{1}{\ " + Taudisplay + "p\ }$";
									"fillColor=blue";
									"shadow=true"];									
			sciblk.graphics.style = strcat(style_properties,";");
			
							
		case "define" then 
	
			Tau=1;		// Constante de temps de l'intégrateur(s)
			TDesat=1	// Constante de temps du désaturateur (s)
			x0=0;		// Valeur initiale nulle du filtre passe-bas			
			reinit=1;	// Réinitialisation du filtre passe-bas			
			sature=1;	// Saturation dynamique
			
			model = scicos_model();
			
			model.sim=list("IntSPEC",4)			// Pour appel fonction "c" IntSPEC
						
			model.intyp = [1;1;1;1];
			model.in = [1;1;1;1]; model.in2 = [1;1;1;1]; 		//4 entrées scalaire

			model.outtyp = 1;
			model.out = 1; model.out2 = 1;		//une sortie scalaire

			model.evtin = [1;1] 				// 2 ports d'entrée d'activation
			
			model.state = x0;
			
			model.ipar=[reinit; sature]
			model.rpar=[Tau; TDesat]
			model.blocktype="c"
			
			model.dep_ut = [%t %t];
			
			exprs = [sci2exp(Tau);sci2exp(x0);sci2exp(TDesat);sci2exp(sature)];
			
			sciblk = standard_define([3 3],model,exprs);
			
			style_properties = ["blockWithLabel";
                    "verticalLabelPosition=middle";
                    "verticalAlign=top";
					"fontColor=black";
					"fontSize=15";
                    "displayedLabel=$\frac{1}{\tau.p\ }$";
					"fillColor=blue";
					"shadow=true"];
					
			sciblk.graphics.style = strcat(style_properties,";")	
			
	end
  
endfunction