//------------------------------------------------------------------------------------------------------------------------
// Fonction			:	Repartiteur_Jets
// Auteur			:	PERRICHON Pierre
// Date				:	13/09/2021
// Version			:	1.0
// Sujet			: 	Fonction sélecteur de jets pour la répartition des efforts sur les turbines Pelton de 3 à 6 jets
// Paramétrage		:	
// Autres données	:	Reprise des alorithmes PELT5 et PELT6 de HYDROTUR Version 1.8
//					:	+ rajouts PELTON 1, 2, 3 et 4 jets
// Sorties			:	
// Langage utilisé	:	Scilab
// Remarques		:	
//						
//
//--------------------------------------------------------------------------------------------------------------------------------------
function res=Repartiteur(nj, NJets, iPilote, RepartiteurOnOff)

	select NJets
	
		case 1 then
				// PELTON 1 jet 
				//--------------
				res=1
				
		case 2 then
				// PELTON 2 jets 
				//--------------		
				res=Select2Jets(nj, NJets, iPilote, RepartiteurOnOff)
				
		case 3 then
				// PELTON 3 jets 
				//--------------		
				res=Select3Jets(nj, NJets, iPilote, RepartiteurOnOff)
				
		case 4 then
				// PELTON 4 jets 
				//--------------				
				res=Select4Jets(nj, NJets, iPilote, RepartiteurOnOff)
				
		case 5 then
				// PELTON 5 jets 
				//--------------				
				res=Select5Jets(nj, NJets, iPilote, RepartiteurOnOff)
				
		case 6 then
				// PELTON 6 jets 
				//--------------				
				res=Select6Jets(nj, NJets, iPilote, RepartiteurOnOff)

		else
				set_blockerror(-1)		// provoque une erreur

	end
	
endfunction

//--------------------------------------------------------------------------------------------------------------------------------------
// REPARTITEUR POUR PELTON 2 JETS
//--------------------------------------------------------------------------------------------------------------------------------------
function res=Select2Jets(nj, NJets, iPilote, RepartiteurOnOff)

	ListePilote=1:NJets;	

	NbrJet=nj;

	if (RepartiteurOnOff <> 0) then
		Pilote=modulo(ListePilote+(iPilote-1)-1,length(ListePilote))+1;
		A=Pilote(1);
		B=Pilote(2);
	else
		NbrJet=NJets;
		A=1;
		B=2;
	end

	select NbrJet

		case 1 then				// 1 seul injecteur avec un gain de 2
			res(A)=NJets/1;
			res(B)=0;

		case 2 then				// 2 injecteurs avec un gain de 1
			res(A)=NJets/2;
			res(B)=NJets/2;

	end

endfunction

//--------------------------------------------------------------------------------------------------------------------------------------
// REPARTITEUR POUR PELTON 3 JETS
//--------------------------------------------------------------------------------------------------------------------------------------
function res=Select3Jets(nj, NJets, iPilote, RepartiteurOnOff)

	ListePilote=1:NJets;	

	NbrJet=nj;

	if (RepartiteurOnOff <> 0) then
		Pilote=modulo(ListePilote+(iPilote-1)-1,length(ListePilote))+1;
		A=Pilote(1);
		B=Pilote(2);
		C=Pilote(3);
	else
		NbrJet=NJets;
		A=1;
		B=2;
		C=3;
	end

	select NbrJet,

		case 1 then				// 1 seul injecteur avec un gain de 3
			res(A)=NJets/1;
			res(B)=0;
			res(C)=0;


		case 2 then				// 2 injecteurs avec un gain de 3/2
			res(A)=NJets/2;
			res(B)=NJets/2;
			res(C)=0;


		case 3 then				// 3 injecteurs avec un gain de 1
			res(A)=NJets/3;
			res(B)=NJets/3;
			res(C)=NJets/3;

	end

endfunction

//--------------------------------------------------------------------------------------------------------------------------------------
// REPARTITEUR POUR PELTON 4 JETS
//--------------------------------------------------------------------------------------------------------------------------------------
function res=Select4Jets(nj, NJets, iPilote, RepartiteurOnOff)

	ListePilote=1:NJets;	

	NbrJet=nj;

	if (RepartiteurOnOff <> 0) then
		Pilote=modulo(ListePilote+(iPilote-1)-1,length(ListePilote))+1;
		A=Pilote(1);
		B=Pilote(2);
		C=Pilote(3);
		D=Pilote(4);
	else
		NbrJet=NJets;
		A=1;
		B=2;
		C=3;
		D=4;
	end

	select NbrJet,

		case 1 then				// 1 seul injecteur avec un gain de 4
			res(A)=NJets/1;
			res(B)=0;
			res(C)=0;
			res(D)=0;


		case 2 then				// 2 injecteurs avec un gain de 2
			res(A)=NJets/2;
			res(B)=0;
			res(C)=NJets/2;
			res(D)=0;


		case 3 then				// 3 injecteurs avec un gain de 4/3
			res(A)=NJets/3;
			res(B)=NJets/3;
			res(C)=NJets/3;
			res(D)=0;


		case 4 then				// 4 injecteurs avec un gain de 1.
			res(A)=NJets/4;
			res(B)=NJets/4;
			res(C)=NJets/4;
			res(D)=NJets/4;
		
	end

endfunction


//--------------------------------------------------------------------------------------------------------------------------------------
// REPARTITEUR POUR PELTON 5 JETS
//--------------------------------------------------------------------------------------------------------------------------------------
function res=Select5Jets(nj, NJets, iPilote, RepartiteurOnOff)

	ListePilote=1:NJets;	

	NbrJet=nj;

	if (RepartiteurOnOff <> 0) then
		Pilote=modulo(ListePilote+(iPilote-1)-1,length(ListePilote))+1;
		A=Pilote(1);
		B=Pilote(2);
		C=Pilote(3);
		D=Pilote(4);
		E=Pilote(5);
	else
		NbrJet=NJets;
		A=1;
		B=2;
		C=3;
		D=4;
		E=5;
	end

	select NbrJet,

		case 1 then				// 1 seul injecteur avec un gain de 5
			res(A)=NJets/1;
			res(B)=0;
			res(C)=0;
			res(D)=0;
			res(E)=0;


		case 2 then				// 2 injecteurs avec un gain de 2.5
			res(A)=NJets/2;
			res(B)=0;
			res(C)=NJets/2;
			res(D)=0;
			res(E)=0;


		case 3 then				// 3 injecteurs avec un gain de 5/3
			res(A)=NJets/3;
			res(B)=0;
			res(C)=NJets/3;
			res(D)=0;
			res(E)=NJets/3;


		case 4 then				// 4 injecteurs avec un gain de 1.25
			res(A)=NJets/4;
			res(B)=NJets/4;
			res(C)=NJets/4;
			res(D)=0;
			res(E)=NJets/4;


		case 5 then				// 5 injecteurs avec un gain de 1
			res(A)=NJets/5;
			res(B)=NJets/5;
			res(C)=NJets/5;
			res(D)=NJets/5;
			res(E)=NJets/5;

		
	end

endfunction


//--------------------------------------------------------------------------------------------------------------------------------------
// REPARTITEUR POUR PELTON 6 JETS
//--------------------------------------------------------------------------------------------------------------------------------------
function res=Select6Jets(nj, NJets, iPilote, RepartiteurOnOff)

	ListePilote=1:NJets;	

	NbrJet=nj;
	
	if (RepartiteurOnOff <> 0) then										// PPe le 14.07.2017
		Pilote=modulo(ListePilote+(iPilote-1)-1,length(ListePilote))+1;	// PPe le 12.02.2018 - Fonction rotation circulaire
		A=Pilote(1);
		B=Pilote(2);
		C=Pilote(3);
		D=Pilote(4);
		E=Pilote(5);
		F=Pilote(6);
	else
		NbrJet=NJets;
		A=1;
		B=2;
		C=3;
		D=4;
		E=5;
		F=6;
	end

	select NbrJet

		case 1 then				// 1 seul injecteur avec un gain de 6
			res(A)=NJets/1;
			res(B)=0;
			res(C)=0;
			res(D)=0;
			res(E)=0;
			res(F)=0;


		case 2 then				// 2 injecteurs avec un gain de 3
			res(A)=NJets/2;
			res(B)=0;
			res(C)=0;
			res(D)=NJets/2;
			res(E)=0;
			res(F)=0;


		case 3 then				// 3 injecteurs avec un gain de 2
			res(A)=NJets/3;
			res(B)=0;
			res(C)=NJets/3;
			res(D)=0;
			res(E)=0;
			res(F)=NJets/3;


		case 4 then				// 4 injecteurs avec un gain de 1.5
			res(A)=NJets/4;
			res(B)=NJets/4;
			res(C)=0;
			res(D)=NJets/4;
			res(E)=NJets/4;
			res(F)=0;


		case 5 then				// 5 injecteurs avec un gain de 1.2
			res(A)=NJets/5;
			res(B)=NJets/5;
			res(C)=NJets/5;
			res(D)=NJets/5;
			res(E)=NJets/5;
			res(F)=0;


		case 6 then				// 6 injecteurs avec un gain de 1
			res(A)=NJets/6;
			res(B)=NJets/6;
			res(C)=NJets/6;
			res(D)=NJets/6;
			res(E)=NJets/6;
			res(F)=NJets/6;
		
	end

endfunction

