//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	EdgeTrigger
// Auteur			:	PERRICHON Pierre
// Date				:	29/12/2018
// Version			:	1.0
//
// Sujet			: 	Détecteur de front d'un signal carré d'entrée
//
// Paramétrage		:	Trig=-1 pour détecter les fronts descendants
//						Trig=+1 pour détecter les fronts montants
//						Trig=0  pour détecter les fronts montants et descendants
//						Datation sur option
//
// Autres données	:	Typiquement, EdgeTrigger est utilisé pour détecter des fronts sur le signal d'entrée
//						Il et donc prévu pour être positionné derrière un relais (HYSERESIS Xcos)
//						
// Sorties			:	Signal de synchronisation
//						Possibilité de sortir ou non la datation de l'évènement sur un port de sortie régulier
//						
//						
// Interface IHM	:	EdgeTrigger.sci
// Code objet		;	EdgeTriggerC.c 
//
// Programme de test:	EdgeTrigger.zos, Busy.zcos
//
//------------------------------------------------------------------------------------------------------------------------
#include <math.h>
#include <stdio.h>
#include "scicos_block4.h"
#include "scicos.h"

#include "version.h"
#if SCI_VERSION_MAJOR < 6
#include "MALLOC.h"
#else
#include "sci_malloc.h"
#endif


#define SignalM work[0]
#define Signal work[1]
#define Activation work[2]
#define Date work[3]

#define Upper 1.0			// Valeur front haut
#define Lower 0				// Valeur front bas

void  EdgeTriggerC(scicos_block *Cblk, scicos_flag flag)
{
	static char *Module="EdgeTriggerC";
	
	double *in,*out;
	double *work = NULL;
	int i;
	int Trigger;
	int Datation;
	
	switch (flag) {

		case Initialization: 
			if ((GetWorkPtrs(Cblk) = (double *)MALLOC(6*sizeof(double))) == NULL ) {
						Coserror("%s: Memory allocation problem\n", "LocalMax"); 
						return;
			}
			work = (double *)GetWorkPtrs(Cblk);
			for (i=0; i<6; i++) work[i] = 0.0;
			break;
			
		case OutputUpdate:
		case ReInitialization:
					work = (double *)GetWorkPtrs(Cblk);
					Datation = Cblk->rpar[1];		// Sortir la datation de l'évènement
					if (Datation) *GetRealOutPortPtrs(Cblk,1) = Date;
					break;			
	
		case StateUpdate:
					work = (double *)GetWorkPtrs(Cblk);
					Trigger = Cblk->rpar[0];		// Lecture mode de déclenchement demandé

					SignalM=Signal;				// Mémorisation du signal ) t-1
					Signal = *GetRealInPortPtrs(Cblk,1);	// Lecture du signal d'entrée
					
					// Gestionnaire de front
					//----------------------
					switch (Trigger) {
						
						case -1 :
									if(SignalM>=Upper && Signal<=Lower) Activation=1;
									break;
									
						case 1 :
									if(Signal>=Upper && SignalM<=Lower) Activation=1;
									break;
									
						case 0 :
									if( (Signal>=Upper && SignalM<=Lower) ||
										(SignalM>=Upper && Signal<=Lower) )	Activation=1;
									break;
									
					}
					if(Activation) Date=get_scicos_time(); 		// Enregistrement de la date de l'évènement
					break;
										
		case OutputEventTiming: 
					work =  (double *)GetWorkPtrs(Cblk);
					if (Activation==1) {
										*GetNevOutPtrs(Cblk) =  0; 
										Activation=0;
					}
					break;
		
		case Ending:
					FREE(GetWorkPtrs(Cblk));
					break;				
		
	}
}