//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Memo
// Auteur			:	PERRICHON Pierre
// Date				:	30/08/2018
// Version			:	1.0
//
// Sujet			: 	Entrée retardée
//
// Paramétrage		:	Valeur initiale de la mémoire
//						
//						
//					
//
// Autres données	:	
//						
//						
// Sorties			:	Entrée retardée
//						
//
// Modification		:
// Auteur			:	PERRICHON Pierre
// Date				:	26/08/2021
// Version			:	1.1
//
// Description		!	Vectorisation (prise en compte de la taille du dimensionnement de la mémoire)						
//
//	
// Modification		:
// Auteur			:	PERRICHON Pierre
// Date				:	17/08/2022
// Version			:	1.2
//
// Description		!	Modification de l'état StateUpdate pour éviter un glitch à t=0						
//					
// Interface IHM	:	Memo.sci
// Code objet		;	MemoC.c 
//
// Programme de test:	Memo.zos
//
//------------------------------------------------------------------------------------------------------------------------
#include <math.h>
#include <stdio.h>
#include "scicos_block4.h"
#include "scicos.h"


#include "version.h"
#if SCI_VERSION_MAJOR < 6
#include "MALLOC.h"
#else
#include "sci_malloc.h"
#endif

#include <sciprint.h>


void  MemoC(scicos_block *Cblk, scicos_flag flag)
{
	static char *Module="MemoC";
	
	double *in,*out;
	double *work = NULL;

	int n = GetInPortRows(Cblk,1); 
	int i;
	double time;

	switch (flag) {

		case Initialization:
					// Allocation d'une zône de travail de 5 doubles
					//----------------------------------------------
					if ((GetWorkPtrs(Cblk) = (double *)MALLOC(n*sizeof(double)))== NULL ) {
						Coserror("%s: Memory allocation problem\n", "LocalMax");
						return;
					}					
					work = (double *)GetWorkPtrs(Cblk);
					
					for(i=0; i<n; ++i) work[i] = Cblk->rpar[i];		// Valeur d'initialisation de la mémoire
					
		case ReInitialization:
		case OutputUpdate:
		
					out = GetRealOutPortPtrs(Cblk,1);
					work = (double *)GetWorkPtrs(Cblk);

					for(i=0; i<n; ++i) out[i] = work[i];		// Restitution du retard
					
					break;
	
		case StateUpdate:
					time=get_scicos_time();		// t en unité seconde
					if (time<=0.) break;		// Evite un glitch à t=0 - PPe le 16.08.2022
					
					in = GetRealInPortPtrs(Cblk,1); 		
					work = (double *)GetWorkPtrs(Cblk);
					
					for(i=0; i<n; ++i) work[i] = in[i];		// Valeur temps réel

					break;					
		
		case Ending:
					FREE(GetWorkPtrs(Cblk));
					break;				
		
	}
}