//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	PSB
// Auteur			:	PERRICHON Pierre
// Date				:	29/12/2018
// Version			:	1.0
//
// Sujet			: 	Filtre passe-bas d'ordre 1 : 1/1+tau*p avec réinitialisation et saturations externes
//						
//
// Paramétrage		:	Constante de temps du filtre d'ordre 1 (s)
//						Valeur initiale du passe-bas à t=0
//						Avec ou sans réinitialisation programmable (0 ou 1)
//						Avec ou sans saturations externes (0 ou 1)
//
// Autres données	:	
//						
//						
// Sorties			:	Signal filtré
//						
//						
//						
// Interface IHM	:	PSB.sci
// Code objet		;	PSBC.c 
//
// Programme de test:	PSB.zos
//
//------------------------------------------------------------------------------------------------------------------------
#include <sciprint.h>

#include "Common.h"



void  PSBC(scicos_block *Cblk, scicos_flag flag)
{
	static char *Module="PSBC";
	
	char MesErr[120];
	char *Label;
	int NumBlockErr;
	
	double *in,*in2,*out,*x,*xd;
	
	int i = 0;
	int n = GetInPortRows(Cblk,1); 
	
	double Tau=Cblk->rpar[0];		
	
	int reinit=Cblk->ipar[0];
	int sature=Cblk->ipar[1];
	
	double UL=1.e32,LL=-UL;			// Upper Limiter - Lower Limiter - Limiteurs du filtre PSB l'intégrateur
	int numport;
	
					
	if (sature) {
					numport=2;
					if (reinit) numport=3;
					 
					UL=*GetRealInPortPtrs(Cblk,numport);
					LL=*GetRealInPortPtrs(Cblk,numport+1);
					
					if (LL>UL) {
						//set_block_error(-1); 	// Données d'entrée inappropriées : voir help c_utils
						NumBlockErr=get_block_number();
						sprintf(MesErr,"Module %s - Block %i \nLower limiter LL=%f greater than the Upper UL=%f",Module,NumBlockErr,LL,UL);
						Coserror(MesErr);
					}
	}

	switch(flag) {
		
		case DerivativeState:

			// Evaluation de la fonction C 
			//----------------------------
			in = GetRealInPortPtrs(Cblk,1);  
			xd = GetDerState(Cblk);		
			out = GetRealOutPortPtrs(Cblk,1);		
		
			for(i=0; i<n; ++i) xd[i] = (in[i] - out[i])/Tau;
			
			break;
				
		case ReInitialization:
			// sciprint("\nUL=%f\nLL=%f\n",UL,LL);
		
		case OutputUpdate:
	
	
			// Evaluation de la fonction F
			//----------------------------
			x = GetState(Cblk);
			out = GetRealOutPortPtrs(Cblk,1);

			for(i=0; i<n; ++i) {
							x[i] = Limiteur(x[i],sature,UL,LL); 	// Limitation de la matrice d'état
							out[i]=x[i];
			}
	
			break;
		
		case StateUpdate:
	
			// Evaluation de la fonction Dx
			//-----------------------------
			x = GetState(Cblk);
			in2 = GetRealInPortPtrs(Cblk,2); 
				
			for(i=0; i<n; ++i) 	x[i] = Limiteur(in2[i],sature,UL,LL);
	
			break;
		
	}
}