//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Rampe
// Auteur			:	PERRICHON Pierre
// Date				:	29/12/2018
// Version			:	1.0
//
// Sujet			: 	Rampe intégrateur avec réinitialisation et saturations externes
//						
//
// Paramétrage		:	Constante de temps du filtre d'ordre 1 (s)
//						Valeur initiale du passe-bas à t=0
//						Avec ou sans réinitialisation programmable (0 ou 1)
//						Avec ou sans saturations externes (0 ou 1)
//
// Autres données	:	
//						
//						
// Sorties			:	Rampe avec réinitialisation et saturations appliquées au signal d'entrée
//						
//						
//						
// Interface IHM	:	Rampe.sci
// Code objet		;	RampeC.c 
//
// Programme de test:	Rampe.zos
//
//------------------------------------------------------------------------------------------------------------------------

//#include <math.h>
//#include <stdio.h>

//#include "scicos_block4.h"
//#include "scicos.h"

#include "Common.h"

void  RampeC(scicos_block *Cblk, scicos_flag flag)
{
	static char *Module="RampeC";
	
	char MesErr[120];
	char *Label;
	int NumBlockErr;
	
	int i = 0;
	int n = GetInPortRows(Cblk,1); 
	
	int reinit=Cblk->ipar[0];
	int sature=Cblk->ipar[1];
	double UL,LL;			// Upper Limiter - Lower Limiter - Limiteurs de l'intégrateur

	int numport;
	
	double *in, *in2, *out,*out2,*x,*xd,Tau;
	int sensRampe1,sensRampe2;		// 2 variables pour éviter un bug
	
	
	if (sature) {
					numport=2;
					if (reinit) numport=3;
					
					UL=*GetRealInPortPtrs(Cblk,numport);
					LL=*GetRealInPortPtrs(Cblk,numport+1);
					
					if (LL>UL) {
						//set_block_error(-1); 	// Données d'entrée inappropriées : voir help c_utils
						NumBlockErr=get_block_number();
						Label=Cblk->label;
						sprintf(MesErr,"Module %s - Block %i \nLower limiter LL=%f greater than the Upper UL=%f",Module,NumBlockErr,LL,UL);
						Coserror(MesErr);

					}
	}
  
	switch (flag) {
	
		case DerivativeState:
			// Evaluation de la fonction C 
			//----------------------------
			in = GetRealInPortPtrs(Cblk,1);  
			xd = GetDerState(Cblk);
			out = GetRealOutPortPtrs(Cblk,1);

			Tau=Cblk->rpar[0];		// Temps de rampe inversé
			if (Tau==0.) Tau=1.e-10;
					
			for(i=0; i<n; ++i) {
								sensRampe1=in[i]<out[i] ? -1 : 1;
								xd[i] = sensRampe1/Tau;
			}
			break;
	
		case ReInitialization:
			x = GetState(Cblk);
			out = GetRealOutPortPtrs(Cblk,1);
			
			for(i=0; i<n; ++i) out[i]=Limiteur(x[i],sature,UL,LL);
			break;
			
		
		case OutputUpdate: 
			// Evaluation de la fonction F
			//----------------------------
			in = GetRealInPortPtrs(Cblk,1);  
			x = GetState(Cblk);
			out = GetRealOutPortPtrs(Cblk,1);

			for(i=0; i<n; ++i) {
								x[i] = Limiteur(x[i],sature,UL,LL); 	// Limitation de la matrice d'état
								
								if (out[i]==in[i]) {
													x[i]=in[i];
								}
								else {
										sensRampe2=out[i]>in[i] ? -1 : 1;

										if (sensRampe2==1) {
												x[i]= x[i]<in[i]  ? x[i] : in[i];
										}								
										else  {
												x[i]= (x[i]>in[i]  ? x[i] : in[i]);								
										}
								
										out[i]=x[i];
								}
			}
			break;
	
		case StateUpdate:
			// Evaluation de la fonction Dx
			//-----------------------------
			x = GetState(Cblk);
			in2 = GetRealInPortPtrs(Cblk,2); 
			out = GetRealOutPortPtrs(Cblk,1);
			
			for(i=0; i<n; ++i) {
								x[i] =Limiteur(in2[i],sature,UL,LL);
								out[i]=x[i];
			}
			break;
	} 
}