//------------------------------------------------------------------------------------------------------------------------
// SEGPAL - Bibliothèque de composants XCOS pour HYDROTUR
//
// Fonction			:	Relais_H
// Auteur			:	PERRICHON Pierre
// Date				:	02/09/2021
// Version			:	1.0
//
// Sujet			: 	Relais en parallèle sur un signal d'entrée 1x1
//
// Paramétrage		:	in :  size(1,1) type 1
//						Switch on : tabeau de seuils de dimention nx1
//						Switch off : tabeau de seuils de dimention nx1
//						Choix de polarité si dépassement de seuil haut dimension 1x1
//						Choix de polarité si dépassement de seuil bas dimension 1x1
//						out size (n,1) type 1
//						Possibilité d'échantillonnner la boite de relayage
//						
//						
//						
//
// Autres données	:	
//						
//						
// Sorties			:	Hysteresis programmé sur chacune des entrées
//						
//						
// Interface IHM	:	Relais_H.sci
// Code objet		;	Relais_HC.c 
//
// Programme de test:	Relais_H.zos
//
//------------------------------------------------------------------------------------------------------------------------

#include <math.h>
#include <stdio.h>
#include <sciprint.h>
#include "scicos_block4.h"
#include "scicos.h"

void Relais_HC(scicos_block *Cblk, int flag)
{
	
static char *Module="Relais_HC";
	
	
int nRelais1=GetOparSize(Cblk,1,1); 			// Nombre de relais par ligne
int nRelais2=GetOparSize(Cblk,1,2); 			// Nombre de relais pr colonne
int nRelais;

double *in = GetRealInPortPtrs(Cblk,1);
double *out = GetRealOutPortPtrs(Cblk,1);
double *rpar=GetRparPtrs(Cblk);

double *TH=GetOparPtrs(Cblk,1);
double *TL=GetOparPtrs(Cblk,2);

double t;

int i;

	switch (flag) {

		case Initialization:	
						nRelais=nRelais1*nRelais2;
						for (i=0; i< nRelais; i++) {
								if(in[0]> TL[i] && in[0]<TH[i]) out[i]=rpar[1];
						}
						break;
						
		case ReInitialization:
		case OutputUpdate:	
						 //t=get_scicos_time();
						 //sciprint("Mise à jour...%f\n",t);
						
						nRelais=nRelais1*nRelais2;
						for (i=0; i < nRelais; i++) {
								if (in[0] >= TH[i]) out[i] = rpar[0];
								 else if (in[0] <= TL[i]) out[i] = rpar[1];								 
 						}
						
						break;
	}
}

